# sinepwm.py 
# Eine Sinuswelle durch Pulsweitenmodulation PWM
#
# Time-stamp: <2011-03-15 21:51:02 daniel>
# Daniel DM3DA dm3da@tuxomania.net
# Uwe DE0508
#
# Fuer Python 2.6 
# 
# Gemeinfrei und ohne Garantie.
# Public domain, no warranties. 
#
# Benutzung: Parameter einstellen und dann 
# mit 
#     python sinepwm.py
# aufrufen. 
#

######################################################################
# Hier Parameter angeben
######################################################################
# gewuenschte Frequenz in Hz:
freq = 660 # Empfohlen 500...800 Hz

# Maximale Bitzahl fuer PWM
maxbit = 128 # 8 Bit = 256

# Oberer Amplitudenwert (duty cycle) in Prozent
maxpercent = 98 # z. B. 90 

# Unterer Amplitudenwert (duty cycle) in Prozent
minpercent = 2 # z. B. 10 

######################################################################
# Programmteil. Hier finden sich keine Einstellungen mehr, hoechstens
# noch Programmfehler
######################################################################
import sys
from math import pi, sin, exp, sqrt


# Begruessung
print "// sinepwm.py erstellt eine Sinustabelle fuer Pulsweitenmodulation"
print

# Plausibiltaetstest Minimum und Maximum
if maxpercent <= minpercent:
    print "// Problem mit Amplitude. Bitte maxpercent und minpercent ansehen."

# Die Kurve darf nicht auf 0 oder die maximalen Bitwerte gesetzt werden
if maxpercent == 100:
    maxamp = maxbit - 1
else:
    maxamp = int(round((maxbit  * maxpercent / 100.0))) # Umrechnen von Prozent 
                                                      # in Schrittweite 
if minpercent == 0:
    minamp = 1
else:
    minamp = int(round((maxbit * minpercent / 100.0))) # Umrechnen von Prozent 
                                                      # in Schrittweite 

peaktopeak = maxamp - minamp
print "// Amplitude von", minamp, "bis", maxamp, "(=", peaktopeak, "Schritte)"

nullwert = int(round((minamp + maxamp) / 2.0))
print "// Nullwert liegt auf einer PWM-Schrittweite von", nullwert 

fpeaktopeak = float(peaktopeak)           # Mit Fliesskomma arbeiten

print "// Start"

sample = 6				 # Anzahl sample der Sinusschwingungen

j=0
while j < sample:
    scale = 1/(1+exp(4.0 - (sample-j-1)*1.25))
#    scale = (sample-j) / float(sample+1)
#    scale = scale * scale * exp(scale/20)
    i=1
    while i <= maxbit:
        sine = sin( 2 * pi * i / maxbit)	# Sinuskurve ausrechnen
        sine = sine * fpeaktopeak * scale / 2	# Auf ganze Amplitude skalieren
        sine = sine + nullwert			# Auf Nullwert anheben
        sine = int(round(sine))			# Auf ganze Zahl runden
        print ' ',sine,',\t//',"scale:",round(scale,2)," ",j*maxbit + i
        i = i + 1
    j = j + 1

print "// End"

